<?php

namespace App\Controller\OgRepository\Image;

use App\Controller\OgRepository\AbstractOgRepositoryController;
use App\Dto\Input\DeployImageInput;
use App\Entity\Image;
use App\Entity\ImageImageRepository;
use App\Model\CommandTypes;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class RenameAction extends AbstractOgRepositoryController
{
    /**
     * @throws TransportExceptionInterface
     * @throws ServerExceptionInterface
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     */
    public function __invoke(Image $image): JsonResponse
    {
        $repositories = $image->getImageImageRepositories();

        if ($repositories->count() === 0) {
            return new JsonResponse(data: ['error' => 'Image is not in any repository', 'code' => Response::HTTP_INTERNAL_SERVER_ERROR], status: Response::HTTP_BAD_REQUEST);
        }

        $allGood = true;
        foreach ($repositories as $repository) {
            try {
                $content = $this->createRequest('GET', 'http://'.$repository->getRepository()->getIp(). ':8006/ogrepository/v1/status');
            } catch (TransportExceptionInterface $e) {
                $allGood = false;
                break;
            }
        }

        if (!$allGood) {
            $this->logger->info('Image is not available in all repositories', ['image' => $image->getName()]);
            return new JsonResponse(data: ['error' => 'Image is not available in all repositories', 'code' => Response::HTTP_INTERNAL_SERVER_ERROR], status: Response::HTTP_BAD_REQUEST);
        }

        $image->setVersion($image->getVersion() + 1);
        $this->entityManager->persist($image);

        $conditional = false;

        $latestImageRepo = $this->entityManager->getRepository(ImageImageRepository::class)->findLatestVersion();
        $repoWithImage = $this->entityManager->getRepository(ImageImageRepository::class)->findBy(['image' => $image, 'version' => $latestImageRepo->getVersion()]);


        foreach ($repoWithImage as $repository) {
            $params = [
                'json' => [
                    'ID_img' => $repository->getImageFullsum(),
                    'image_new_name' => $image->getName().'_v'.$image->getVersion(),
                ]
            ];

            $content = $this->createRequest('PUT', 'http://'.$repository->getRepository()->getIp().':8006/ogrepository/v1/images/rename', $params);

            if (isset($content['error']) && $content['code'] === Response::HTTP_INTERNAL_SERVER_ERROR ) {
                $conditional = true;
            }
        }

        if ($conditional) {
            return new JsonResponse(data: ['error' => 'Error renaming image'], status: Response::HTTP_INTERNAL_SERVER_ERROR);
        }

        $this->entityManager->flush();

        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}